@@ -66,7 +66,8 @@ write.graph.fromraw <- function(buffer, file) {
   invisible(NULL)
 }
 
-read.graph <- function(file, format="edgelist", ...) {
+read.graph <- function(file, format=c("edgelist", "pajek", "ncol", "lgl",
+                               "graphml", "dimacs", "graphdb", "gml"), ...) {
 
   if (igraph.i.have.fmemopen) {
     file <- read.graph.toraw(file)
@@ -78,8 +79,7 @@ read.graph <- function(file, format="edgelist", ...) {
     }
   }
 
-  format <- tolower(format)
-  
+  format <- igraph.match.arg(format)
   res <- switch(format,
                 "pajek"=read.graph.pajek(file, ...),
                 "ncol"=read.graph.ncol(file, ...),
@@ -94,7 +94,8 @@ read.graph <- function(file, format="edgelist", ...) {
   res
 }
 
-write.graph <- function(graph, file, format="edgelist", ...) {
+write.graph <- function(graph, file, format=c("edgelist", "pajek", "ncol", "lgl",
+                                       "graphml", "dimacs", "gml", "dot"), ...) {
 
   if (!is.igraph(graph)) {
     stop("Not a graph object")
@@ -109,8 +110,7 @@ write.graph <- function(graph, file, format="edgelist", ...) {
     }
   }
   
-  format <- tolower(format)
-
+  format <- igraph.match.arg(format)
   res <- switch(format,
                 "pajek"=write.graph.pajek(graph, file, ...),
                 "edgelist"=write.graph.edgelist(graph, file, ...),
@@ -119,6 +119,7 @@ write.graph <- function(graph, file, format="edgelist", ...) {
                 "graphml"=write.graph.graphml(graph, file, ...),
                 "dimacs"=write.graph.dimacs(graph, file, ...),
                 "gml"=write.graph.gml(graph, file, ...),
+                "dot"=write.graph.dot(graph, file, ...),
                 stop(paste("Unknown file format:",format))
                 )
 
@@ -141,6 +142,7 @@ write.graph <- function(graph, file, format="edgelist", ...) {
 read.graph.edgelist <- function(file, n=0,
                                 directed=TRUE, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_edgelist", file,
         as.numeric(n), as.logical(directed),
         PACKAGE="igraph")
@@ -149,6 +151,7 @@ read.graph.edgelist <- function(file, n=0,
 write.graph.edgelist <- function(graph, file, 
                                  ...) {
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_edgelist", graph, file,
         PACKAGE="igraph")
 }
@@ -160,6 +163,7 @@ write.graph.edgelist <- function(graph, file,
 read.graph.ncol <- function(file, predef=character(0), names=TRUE,
                            weights=TRUE, directed=FALSE, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_ncol", file, as.character(predef),
         as.logical(names), as.logical(weights), as.logical(directed),
         PACKAGE="igraph")
@@ -172,6 +176,7 @@ write.graph.ncol <- function(graph, file,
   if (length(names)==0 || ! names %in% list.vertex.attributes(graph)) { names <- NULL }
   if (length(weights)==0 || ! weights %in% list.edge.attributes(graph)) { weights <- NULL }
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_ncol", graph, file,
         names, weights,
         PACKAGE="igraph")
@@ -180,6 +185,7 @@ write.graph.ncol <- function(graph, file,
 read.graph.lgl <- function(file, names=TRUE,
                            weights=TRUE, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_lgl", file,
         as.logical(names), as.logical(weights),
         PACKAGE="igraph")
@@ -193,6 +199,7 @@ write.graph.lgl <- function(graph, file,
   if (length(names)==0 || ! names %in% list.vertex.attributes(graph)) { names <- NULL }
   if (length(weights)==0 || ! weights %in% list.edge.attributes(graph)) { weights <- NULL }
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_lgl", graph, file,
         names, weights, as.logical(isolates),
         PACKAGE="igraph")
@@ -200,12 +207,14 @@ write.graph.lgl <- function(graph, file,
 
 read.graph.pajek <- function(file, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_pajek", file,
         PACKAGE="igraph")
 }
 
 write.graph.pajek <- function(graph, file, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_pajek", graph, file,
         PACKAGE="igraph")
 }
@@ -214,11 +223,19 @@ read.graph.dimacs <- function(file, directed=TRUE, ...) {
 
   res <- .Call("R_igraph_read_graph_dimacs", file, as.logical(directed),
                PACKAGE="igraph")
-  graph <- res[[1]]
-  graph <- set.graph.attribute(graph, "source", res[[2]])
-  graph <- set.graph.attribute(graph, "target", res[[3]])
-  E(graph)$capacity <- res[[4]]
-  graph
+  if (res[[1]][1] == "max") {
+    graph <- res[[2]]
+    graph <- set.graph.attribute(graph, "problem", res[[1]])
+    graph <- set.graph.attribute(graph, "source", res[[3]])
+    graph <- set.graph.attribute(graph, "target", res[[4]])
+    E(graph)$capacity <- res[[5]]
+    graph
+  } else if (res[[1]][1] == "edge") {
+    graph <- res[[2]]
+    graph <- set.graph.attribute(graph, "problem", res[[1]])
+    V(graph)$label <- res[[3]]
+    graph
+  }
 }
 
 write.graph.dimacs <- function(graph, file,
@@ -234,6 +251,7 @@ write.graph.dimacs <- function(graph, file,
     capacity <- E(graph)$capacity
   }
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_dimacs", graph, file, as.numeric(source),
         as.numeric(target), as.numeric(capacity),
         PACKAGE="igraph")
@@ -245,12 +263,14 @@ write.graph.dimacs <- function(graph, file,
 
 read.graph.graphml <- function(file, index=0, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_graphml", file, as.numeric(index),
         PACKAGE="igraph")
 }
 
 write.graph.graphml <- function(graph, file, ...) {
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_graphml", graph, file,
         PACKAGE="igraph")
 }
@@ -260,6 +280,7 @@ write.graph.graphml <- function(graph, file, ...) {
 ################################################################
 
 read.graph.gml <- function(file, ...) {
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_gml", file,
         PACKAGE="igraph")
 }
@@ -271,10 +292,21 @@ write.graph.gml <- function(graph, file, id=NULL, creator=NULL, ...) {
   if (!is.null(creator)) {
     creator <- as.character(creator)
   }
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_write_graph_gml", graph, file, id, creator,
         PACKAGE="igraph")
 }
 
+################################################################
+# Dot
+################################################################
+
+write.graph.dot <- function(graph, file, ...) {
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
+  .Call("R_igraph_write_graph_dot", graph, file,
+        PACKAGE="igraph")
+}
+
 ################################################################
 # Download a file from the graph database for
 # isomorphic problems
@@ -338,6 +370,7 @@ graph.graphdb <- function(url=NULL,
 }
 
 read.graph.graphdb <- function(file, directed=TRUE, ...) {
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_read_graph_graphdb", file, as.logical(directed),
         PACKAGE="igraph")
 }
