@@ -522,3 +522,159 @@ piecewise.layout <- function(graph, layout=layout.kamada.kawai, ...) {
   l[ unlist(sapply(gl, get.vertex.attribute, "id")), ] <- l[]
   l
 }
+
+layout.drl <- function(graph, use.seed = FALSE,
+                       seed=matrix(runif(vcount(graph)*2), nc=2),
+                       options=igraph.drl.default,
+                       weights=E(graph)$weight,
+                       fixed=NULL) 
+{
+    if (!is.igraph(graph)) {
+        stop("Not a graph object")
+    }
+    use.seed <- as.logical(use.seed)
+    seed <- as.matrix(seed)
+    options.tmp <- igraph.drl.default
+    options.tmp[names(options)] <- options
+    options <- options.tmp
+    if (!is.null(weights)) {
+      weights <- as.numeric(weights)
+    }
+    if (!is.null(fixed)) {
+      fixed <- as.logical(fixed)
+    }
+    on.exit(.Call("R_igraph_finalizer", PACKAGE = "igraph"))
+    res <- .Call("R_igraph_layout_drl", graph, seed, use.seed, options, 
+                 weights, fixed, PACKAGE = "igraph")
+    res
+}
+
+igraph.drl.default <- list(edge.cut=32/40,
+                           init.iterations=0,
+                           init.temperature=2000,
+                           init.attraction=10,
+                           init.damping.mult=1.0,
+                           liquid.iterations=200,
+                           liquid.temperature=2000,
+                           liquid.attraction=10,
+                           liquid.damping.mult=1.0,
+                           expansion.iterations=200,
+                           expansion.temperature=2000,
+                           expansion.attraction=2,
+                           expansion.damping.mult=1.0,
+                           cooldown.iterations=200,
+                           cooldown.temperature=2000,
+                           cooldown.attraction=1,
+                           cooldown.damping.mult=.1,
+                           crunch.iterations=50,
+                           crunch.temperature=250,
+                           crunch.attraction=1,
+                           crunch.damping.mult=0.25,
+                           simmer.iterations=100,
+                           simmer.temperature=250,
+                           simmer.attraction=.5,
+                           simmer.damping.mult=0)
+
+igraph.drl.coarsen <- list(edge.cut=32/40,
+                           init.iterations=0,
+                           init.temperature=2000,
+                           init.attraction=10,
+                           init.damping.mult=1.0,
+                           liquid.iterations=200,
+                           liquid.temperature=2000,
+                           liquid.attraction=2,
+                           liquid.damping.mult=1.0,
+                           expansion.iterations=200,
+                           expansion.temperature=2000,
+                           expansion.attraction=10,
+                           expansion.damping.mult=1.0,
+                           cooldown.iterations=200,
+                           cooldown.temperature=2000,
+                           cooldown.attraction=1,
+                           cooldown.damping.mult=.1,
+                           crunch.iterations=50,
+                           crunch.temperature=250,
+                           crunch.attraction=1,
+                           crunch.damping.mult=0.25,
+                           simmer.iterations=100,
+                           simmer.temperature=250,
+                           simmer.attraction=.5,
+                           simmer.damping.mult=0)
+
+igraph.drl.coarsest <- list(edge.cut=32/40,
+                            init.iterations=0,
+                            init.temperature=2000,
+                            init.attraction=10,
+                            init.damping.mult=1.0,
+                            liquid.iterations=200,
+                            liquid.temperature=2000,
+                            liquid.attraction=2,
+                            liquid.damping.mult=1.0,
+                            expansion.iterations=200,
+                            expansion.temperature=2000,
+                            expansion.attraction=10,
+                            expansion.damping.mult=1.0,
+                            cooldown.iterations=200,
+                            cooldown.temperature=2000,
+                            cooldown.attraction=1,
+                            cooldown.damping.mult=.1,
+                            crunch.iterations=200,
+                            crunch.temperature=250,
+                            crunch.attraction=1,
+                            crunch.damping.mult=0.25,
+                            simmer.iterations=100,
+                            simmer.temperature=250,
+                            simmer.attraction=.5,
+                            simmer.damping.mult=0)
+
+igraph.drl.refine <- list(edge.cut=32/40,
+                          init.iterations=0,
+                          init.temperature=50,
+                          init.attraction=.5,
+                          init.damping.mult=1.0,
+                          liquid.iterations=0,
+                          liquid.temperature=2000,
+                          liquid.attraction=2,
+                          liquid.damping.mult=1.0,
+                          expansion.iterations=50,
+                          expansion.temperature=500,
+                          expansion.attraction=.1,
+                          expansion.damping.mult=.25,
+                          cooldown.iterations=50,
+                          cooldown.temperature=250,
+                          cooldown.attraction=1,
+                          cooldown.damping.mult=.1,
+                          crunch.iterations=50,
+                          crunch.temperature=250,
+                          crunch.attraction=1,
+                          crunch.damping.mult=0.25,
+                          simmer.iterations=0,
+                          simmer.temperature=250,
+                          simmer.attraction=.5,
+                          simmer.damping.mult=0)
+
+igraph.drl.final <- list(edge.cut=32/40,
+                         init.iterations=0,
+                         init.temperature=50,
+                         init.attraction=.5,
+                         init.damping.mult=0,
+                         liquid.iterations=0,
+                         liquid.temperature=2000,
+                         liquid.attraction=2,
+                         liquid.damping.mult=1.0,
+                         expansion.iterations=50,
+                         expansion.temperature=2000,
+                         expansion.attraction=2,
+                         expansion.damping.mult=1.0,
+                         cooldown.iterations=50,
+                         cooldown.temperature=200,
+                         cooldown.attraction=1,
+                         cooldown.damping.mult=.1,
+                         crunch.iterations=50,
+                         crunch.temperature=250,
+                         crunch.attraction=1,
+                         crunch.damping.mult=0.25,
+                         simmer.iterations=25,
+                         simmer.temperature=250,
+                         simmer.attraction=.5,
+                         simmer.damping.mult=0)
