@@ -25,6 +25,7 @@ evolver.d <- function(nodes, kernel, outseq=NULL, outdist=NULL,
 
   if (!is.null(outseq)) { outseq <- as.numeric(outseq) }
   if (!is.null(outdist)) { outdist <- as.numeric(outdist) }
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_evolver_d", as.numeric(nodes), as.numeric(kernel),
         outseq, outdist, m, as.logical(directed), as.logical(verbose),
         PACKAGE="igraph")
@@ -38,6 +39,7 @@ revolver.d <- function(graph, niter=5, sd=FALSE, norm=FALSE,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_d", graph, as.numeric(niter), as.logical(sd),
         as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
@@ -50,6 +52,7 @@ revolver.error.d <- function(graph, kernel) {
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_d", graph, as.numeric(kernel),
         PACKAGE="igraph")
 }
@@ -61,6 +64,7 @@ revolver.ad <- function(graph, niter=5, agebins=max(vcount(graph)/7100, 10),
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_ad", graph, as.numeric(niter), as.numeric(agebins),
         as.logical(sd), as.logical(norm), as.logical(cites),
         as.logical(expected), as.logical(error),
@@ -76,6 +80,7 @@ revolver.error.ad <- function(graph, kernel) {
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_ad", graph, kernel,
         PACKAGE="igraph")
 }
@@ -88,6 +93,7 @@ revolver.ade <- function(graph, cats, niter=5, agebins=max(vcount(graph)/7100, 1
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_ade", graph, as.numeric(cats), as.numeric(niter),
         as.numeric(agebins), as.logical(sd), as.logical(norm), as.logical(cites),
         as.logical(expected), as.logical(error),
@@ -102,18 +108,20 @@ revolver.error.ade <- function(graph, kernel, cats) {
   }
   
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_ade", graph, kernel, as.numeric(cats),
         PACKAGE="igraph")
 }
 
-revolver.e <- function(graph, cats, niter=5,
+revolver.e <- function(graph, cats, niter=5, st=FALSE,
                       sd=FALSE, norm=FALSE, cites=FALSE, expected=FALSE,
                       error=TRUE, debug=numeric(), verbose=igraph.par("verbose")) {
   if (!is.igraph(graph)) {
     stop("Not a graph object!")
   }
 
-  .Call("R_igraph_revolver_e", graph, as.numeric(cats), as.numeric(niter),
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
+  .Call("R_igraph_revolver_e", graph, as.numeric(cats), as.numeric(niter), as.logical(st),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
         PACKAGE="igraph")
@@ -125,6 +133,7 @@ revolver.error.e <- function(graph, kernel, cats) {
     stop("Not a graph object!")
   }
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_e", graph, as.numeric(kernel), as.numeric(cats),
         PACKAGE="igraph")
 }
@@ -136,6 +145,7 @@ revolver.de <- function(graph, cats, niter=5,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_de", graph, as.numeric(cats), as.numeric(niter),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
@@ -149,6 +159,7 @@ revolver.error.de <- function(graph, kernel, cats) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_de", graph, kernel, as.numeric(cats),
         PACKAGE="igraph")
 }
@@ -160,6 +171,7 @@ revolver.l <- function(graph, niter=5, agebins=max(vcount(graph)/7100, 10),
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_l", graph, as.numeric(niter), as.numeric(agebins),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
@@ -172,6 +184,7 @@ revolver.error.l <- function(graph, kernel) {
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_l", graph, as.numeric(kernel),
         PACKAGE="igraph")
 }
@@ -183,6 +196,7 @@ revolver.dl <- function(graph, niter=5, agebins=max(vcount(graph)/7100, 10),
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_dl", graph, as.numeric(niter), as.numeric(agebins),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
@@ -196,6 +210,7 @@ revolver.error.dl <- function(graph, kernel) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_dl", graph, kernel,
         PACKAGE="igraph")
 }
@@ -207,6 +222,7 @@ revolver.el <- function(graph, cats, niter=5, agebins=max(vcount(graph)/7100, 10
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_el", graph, as.numeric(cats), as.numeric(niter),
         as.numeric(agebins),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
@@ -221,6 +237,7 @@ revolver.error.el <- function(graph, kernel, cats) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_el", graph, kernel, as.numeric(cats),
         PACKAGE="igraph")
 }
@@ -233,6 +250,7 @@ revolver.r <- function(graph, window, niter=5, sd=FALSE, norm=FALSE,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_r", graph, as.numeric(niter), as.numeric(window),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
@@ -245,6 +263,7 @@ revolver.error.r <- function(graph, kernel, window) {
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_r", graph, as.numeric(kernel), as.numeric(window),
         PACKAGE="igraph")
 }
@@ -256,6 +275,7 @@ revolver.ar <- function(graph, window, niter=5, agebins=max(vcount(graph)/7100,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_ar", graph, as.numeric(niter), as.numeric(agebins),
         as.numeric(window),
         as.logical(sd), as.logical(norm), as.logical(cites),
@@ -271,6 +291,7 @@ revolver.error.ar <- function(graph, kernel, window) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_ar", graph, kernel, as.numeric(window),
         PACKAGE="igraph")
 }
@@ -282,6 +303,7 @@ revolver.di <- function(graph, cats, niter=5,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_di", graph, as.numeric(cats), as.numeric(niter),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
         as.logical(error), as.numeric(debug), as.logical(verbose),
@@ -295,6 +317,7 @@ revolver.error.di <- function(graph, kernel, cats) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_di", graph, kernel, as.numeric(cats),
         PACKAGE="igraph")
 }
@@ -307,6 +330,7 @@ revolver.adi <- function(graph, cats, niter=5, agebins=max(vcount(graph)/7100, 1
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_adi", graph, as.numeric(cats), as.numeric(niter),
         as.numeric(agebins), as.logical(sd), as.logical(norm), as.logical(cites),
         as.logical(expected), as.logical(error),
@@ -321,6 +345,7 @@ revolver.error.adi <- function(graph, kernel, cats) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_adi", graph, kernel, as.numeric(cats),
         PACKAGE="igraph")
 }
@@ -332,6 +357,7 @@ revolver.il <- function(graph, cats, niter=5, agebins=max(vcount(graph)/7100, 10
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_il", graph, as.numeric(cats),
         as.numeric(niter), as.numeric(agebins),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
@@ -346,6 +372,7 @@ revolver.error.il <- function(graph, kernel, cats) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_il", graph, kernel, as.numeric(cats),
         PACKAGE="igraph")
 }
@@ -357,6 +384,7 @@ revolver.ir <- function(graph, cats, window, niter=5,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_ir", graph, as.numeric(cats), as.numeric(window),
         as.numeric(niter),
         as.logical(sd), as.logical(norm), as.logical(cites), as.logical(expected),
@@ -371,6 +399,7 @@ revolver.error.ir <- function(graph, kernel, cats, window) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_ir", graph, kernel, as.numeric(cats),
         as.numeric(window),
         PACKAGE="igraph")
@@ -385,6 +414,7 @@ revolver.air <- function(graph, cats, window,
     stop("Not a graph object!")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_air", graph, as.numeric(cats), as.numeric(window),
         as.numeric(niter),
         as.numeric(agebins), as.logical(sd), as.logical(norm), as.logical(cites),
@@ -400,6 +430,7 @@ revolver.error.air <- function(graph, kernel, cats, window) {
   }
 
   kernel <- structure(as.numeric(kernel), dim=dim(kernel))
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_error2_air", graph, kernel, as.numeric(cats),
         as.numeric(window),
         PACKAGE="igraph")
@@ -420,6 +451,7 @@ revolver.d.d <- function(graph, vtime=V(graph)$time, etime=E(graph)$time, niter=
     stop("etime missing")
   }
 
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_d_d", graph, as.numeric(niter),
         as.numeric(vtime), as.numeric(etime), 
         as.logical(sd), as.logical(norm), as.logical(cites),
@@ -451,6 +483,7 @@ revolver.p.p <- function(graph, events=get.graph.attribute(graph, "events"),
   authors <- unlist(events)
   eventsizes <- sapply(events, length)
   
+  on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_revolver_p_p", graph, as.numeric(niter),
         as.numeric(vtime), as.numeric(etime), as.numeric(authors),
         as.numeric(eventsizes), as.logical(sd), as.logical(norm), as.logical(cites),
