@@ -112,7 +112,8 @@ closeness <- function(graph, v=V(graph), mode=c("all", "out", "in")) {
         PACKAGE="igraph")
 }
 
-shortest.paths <- function(graph, v=V(graph), mode=c("all", "out", "in")) {
+shortest.paths <- function(graph, v=V(graph), mode=c("all", "out", "in"),
+                           weights=NULL) {
 
   if (!is.igraph(graph)) {
     stop("Not a graph object")
@@ -120,14 +121,27 @@ shortest.paths <- function(graph, v=V(graph), mode=c("all", "out", "in")) {
   mode <- igraph.match.arg(mode)
   mode <- switch(mode, "out"=1, "in"=2, "all"=3)
 
+  if (is.null(weights)) {
+    if ("weight" %in% list.edge.attributes(graph)) {
+      weights <- as.numeric(E(graph)$weight)
+    }
+  } else {
+    if (length(weights)==1 && is.na(weights)) {
+      weights <- NULL
+    } else {
+      weights <- as.numeric(weights)
+    }
+  }
+
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_shortest_paths", graph, as.igraph.vs(v),
-        as.numeric(mode),
+        as.numeric(mode), weights,
         PACKAGE="igraph")
 }
 
 get.shortest.paths <- function(graph, from, to=V(graph),
-                               mode=c("all", "out", "in")) {
+                               mode=c("all", "out", "in"),
+                               weights=NULL) {
 
   if (!is.igraph(graph)) {
     stop("Not a graph object")
@@ -135,11 +149,23 @@ get.shortest.paths <- function(graph, from, to=V(graph),
   mode <- igraph.match.arg(mode)
   mode <- switch(mode, "out"=1, "in"=2, "all"=3)
 
+  if (is.null(weights)) {
+    if ("weight" %in% list.edge.attributes(graph)) {
+      weights <- as.numeric(E(graph)$weight)
+    }
+  } else {
+    if (length(weights)==1 && is.na(weights)) {
+      weights <- NULL
+    } else {
+      weights <- as.numeric(weights)
+    }
+  }
+  
   to <- as.igraph.vs(to)
   on.exit( .Call("R_igraph_finalizer", PACKAGE="igraph") )
   .Call("R_igraph_get_shortest_paths", graph,
         as.numeric(from), to, as.numeric(mode), as.numeric(length(to)),
-        PACKAGE="igraph")
+        weights, PACKAGE="igraph")
 }
 
 get.all.shortest.paths <- function(graph, from,
