@@ -471,7 +471,9 @@ int igraph_i_fastgreedy_commpair_cmp(const void* p1, const void* p2) {
  * structure in mega-scale social networks,
  * http://www.arxiv.org/abs/cs.CY/0702048v1 have also been implemented.
  *
- * \param graph The input graph.
+ * \param graph The input graph. It must be a simple graph, i.e. a graph 
+ *    without multiple and without loop edges. This is checked and an
+ *    error message is given for non-simple graphs.
  * \param merges Pointer to an initialized matrix or NULL, the result of the
  *    computation is stored here. The matrix has two columns and each
  *    merge corresponds to one merge, the ids of the two merged
@@ -507,6 +509,7 @@ int igraph_community_fastgreedy(const igraph_t *graph,
   igraph_i_fastgreedy_community_list communities;
   igraph_vector_t a;
   igraph_real_t q, maxq, *dq, weight_sum;
+  igraph_bool_t simple;
 
   /*long int join_order[] = { 16,5, 5,6, 6,0, 4,0, 10,0, 26,29, 29,33, 23,33, 27,33, 25,24, 24,31, 12,3, 21,1, 30,8, 8,32, 9,2, 17,1, 11,0, 7,3, 3,2, 13,2, 1,2, 28,31, 31,33, 22,32, 18,32, 20,32, 32,33, 15,33, 14,33, 0,19, 19,2, -1,-1 };*/
   /*long int join_order[] = { 43,42, 42,41, 44,41, 41,36, 35,36, 37,36, 36,29, 38,29, 34,29, 39,29, 33,29, 40,29, 32,29, 14,29, 30,29, 31,29, 6,18, 18,4, 23,4, 21,4, 19,4, 27,4, 20,4, 22,4, 26,4, 25,4, 24,4, 17,4, 0,13, 13,2, 1,2, 11,2, 8,2, 5,2, 3,2, 10,2, 9,2, 7,2, 2,28, 28,15, 12,15, 29,16, 4,15, -1,-1 };*/
@@ -520,13 +523,6 @@ int igraph_community_fastgreedy(const igraph_t *graph,
   
   total_joins=no_of_nodes-1;
 
-  if (merges != 0) {
-	IGRAPH_CHECK(igraph_matrix_resize(merges, total_joins, 2));
-	igraph_matrix_null(merges);
-  }
-  if (modularity != 0) {
-	IGRAPH_CHECK(igraph_vector_resize(modularity, total_joins+1));
-  }
   if (weights != 0) {
     if (igraph_vector_size(weights) < igraph_ecount(graph))
       IGRAPH_ERROR("fast greedy community detection: weight vector too short", IGRAPH_EINVAL);
@@ -535,6 +531,19 @@ int igraph_community_fastgreedy(const igraph_t *graph,
     weight_sum = igraph_vector_sum(weights);
   } else weight_sum = no_of_edges;
 
+  IGRAPH_CHECK(igraph_is_simple(graph, &simple));
+  if (!simple) {
+    IGRAPH_ERROR("fast-greedy community finding works only on simple graphs", IGRAPH_EINVAL);
+  }
+
+  if (merges != 0) {
+	IGRAPH_CHECK(igraph_matrix_resize(merges, total_joins, 2));
+	igraph_matrix_null(merges);
+  }
+  if (modularity != 0) {
+	IGRAPH_CHECK(igraph_vector_resize(modularity, total_joins+1));
+  }
+
   /* Create degree vector */
   IGRAPH_VECTOR_INIT_FINALLY(&a, no_of_nodes);
   if (weights) {
