@@ -1,4 +1,3 @@
-
 /* -*- mode: C -*-  */
 /* 
    IGraph R library.
@@ -281,23 +280,26 @@ int igraph_erdos_renyi_game_gnm(igraph_t *graph, igraph_integer_t n, igraph_real
       retval=igraph_full(graph, n, directed, loops);
     } else {
     
+      long int slen;
+      
       IGRAPH_VECTOR_INIT_FINALLY(&s, 0);
       IGRAPH_CHECK(igraph_random_sample(&s, 0, maxedges-1, no_of_edges));
       
       IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
       IGRAPH_CHECK(igraph_vector_reserve(&edges, igraph_vector_size(&s)*2));
-      
+
+      slen=igraph_vector_size(&s);
       if (directed && loops) {
-	for (i=0; i<igraph_vector_size(&s); i++) {
+	for (i=0; i<slen; i++) {
 	  long int to=floor(VECTOR(s)[i]/no_of_nodes);
 	  long int from=VECTOR(s)[i]-((igraph_real_t)to)*no_of_nodes;
 	  igraph_vector_push_back(&edges, from);
 	  igraph_vector_push_back(&edges, to);
 	}
       } else if (directed && !loops) {
-	for (i=0; i<igraph_vector_size(&s); i++) {
-	  long int to=floor(VECTOR(s)[i]/no_of_nodes);
-	  long int from=VECTOR(s)[i]-((igraph_real_t)to)*no_of_nodes;
+	for (i=0; i<slen; i++) {
+	  long int from=floor(VECTOR(s)[i]/(no_of_nodes-1));
+	  long int to=VECTOR(s)[i]-((igraph_real_t)from)*(no_of_nodes-1);
 	  if (from==to) {
 	    to=no_of_nodes-1;
 	  }
@@ -305,14 +307,14 @@ int igraph_erdos_renyi_game_gnm(igraph_t *graph, igraph_integer_t n, igraph_real
 	  igraph_vector_push_back(&edges, to);
 	}
       } else if (!directed && loops) {
-	for (i=0; i<igraph_vector_size(&s); i++) {
+	for (i=0; i<slen; i++) {
 	  long int to=floor((sqrt(8*VECTOR(s)[i]+1)-1)/2);
 	  long int from=VECTOR(s)[i]-(((igraph_real_t)to)*(to+1))/2;
 	  igraph_vector_push_back(&edges, from);
 	  igraph_vector_push_back(&edges, to);
 	}
       } else /* !directed && !loops */ {
-	for (i=0; i<igraph_vector_size(&s); i++) {
+	for (i=0; i<slen; i++) {
 	  long int to=floor((sqrt(8*VECTOR(s)[i]+1)+1)/2);
 	  long int from=VECTOR(s)[i]-(((igraph_real_t)to)*(to-1))/2;
 	  igraph_vector_push_back(&edges, from);
@@ -497,6 +499,11 @@ int igraph_degree_sequence_game_simple(igraph_t *graph,
   return 0;
 }
 
+/* This is in gengraph_mr-connected.cpp */
+
+int igraph_degree_sequence_game_vl(igraph_t *graph,
+				   const igraph_vector_t *out_seq,
+				   const igraph_vector_t *in_seq);
 /**
  * \ingroup generators
  * \function igraph_degree_sequence_game
@@ -518,6 +525,13 @@ int igraph_degree_sequence_game_simple(igraph_t *graph,
  *        generate both loop (self) edges and multiple edges.
  *        For directed graphs, the algorithm is basically the same,
  *        but two separate bags are used for the in- and out-degrees. 
+ *        \c IGRAPH_DEGSEQ_VL is a much more sophisticated generator, 
+ *        that can sample undirected, connected simple graphs uniformly. 
+ *        It uses Monte-Carlo methods to randomize the graphs. 
+ *        This generator should be favoured if undirected and connected 
+ *        graphs are to be generated. igraph uses the original implementation 
+ *        Fabien Viger; see http://www-rp.lip6.fr/~latapy/FV/generation.html
+ *        and the paper cited on it for the details of the algorithm.
  * \return Error code: 
  *          \c IGRAPH_ENOMEM: there is not enough
  *           memory to perform the operation.
@@ -543,6 +557,8 @@ int igraph_degree_sequence_game(igraph_t *graph, const igraph_vector_t *out_deg,
 
   if (method==IGRAPH_DEGSEQ_SIMPLE) {
     retval=igraph_degree_sequence_game_simple(graph, out_deg, in_deg);
+  } else if (method==IGRAPH_DEGSEQ_VL) {
+    retval=igraph_degree_sequence_game_vl(graph, out_deg, in_deg);
   } else {
     IGRAPH_ERROR("Invalid degree sequence game method", IGRAPH_EINVAL);
   }
@@ -1531,13 +1547,26 @@ int igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
   
   return 0;
 }
-  
+
+
+void igraph_i_preference_game_free_vids_by_type(igraph_vector_ptr_t *vecs) {
+  int i=0, n;
+  igraph_vector_t *v;
+
+  n = igraph_vector_ptr_size(vecs);
+  for (i=0; i<n; i++) {
+    v = (igraph_vector_t*)VECTOR(*vecs)[i];
+	if (v) igraph_vector_destroy(v);
+  }
+  igraph_vector_ptr_destroy_all(vecs);
+}
+
 /**
  * \function igraph_preference_game
  * \brief Generates a graph with vertex types and connection preferences 
  * 
  * </para><para>
- * This is practically the nongrowing variant of \c igraph_establishment_game .
+ * This is practically the nongrowing variant of \ref igraph_establishment_game .
  * A given number of vertices are generated. Every vertex is assigned to a
  * vertex type according to the given type probabilities. Finally, every
  * vertex pair is evaluated and an edge is created between them with a
@@ -1546,11 +1575,13 @@ int igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
  * \param graph Pointer to an uninitialized graph.
  * \param nodes The number of vertices in the graph.
  * \param types The number of vertex types.
- * \param type_dist Vector giving the distribution of vertex types.
+ * \param type_dist Vector giving the distribution of vertex types. If \c NULL ,
+ *   all vertex types will have equal probability.
  * \param pref_matrix Matrix giving the connection probabilities for
- *   different vertex types.
+ *   different vertex types. This should be symmetric if the requested
+ *   graph is undirected.
  * \param node_type_vec A vector where the individual generated vertex types
- *   will be stored. If NULL, the vertex types won't be saved.
+ *   will be stored. If \c NULL , the vertex types won't be saved.
  * \param directed Logical, whether to generate a directed graph. If undirected
  *   graphs are requested, only the lower left triangle of the preference
  *   matrix is considered.
@@ -1559,8 +1590,9 @@ int igraph_grg_game(igraph_t *graph, igraph_integer_t nodes,
  *
  * Added in version 0.3.</para><para>
  *
- * Time complexity: O(|V|^2), |V| is the number of vertices.
- *
+ * Time complexity: O(|V|+|E|), the
+ * number of vertices plus the number of edges in the graph.
+ * 
  * \sa igraph_establishment_game()
  */
 
@@ -1573,12 +1605,24 @@ int igraph_preference_game(igraph_t *graph, igraph_integer_t nodes,
 			   igraph_bool_t loops) {
   
   long int i, j;
-  igraph_vector_t edges;
+  igraph_vector_t edges, s;
   igraph_vector_t cumdist;
   igraph_vector_t* nodetypes;
-  igraph_real_t maxcum;
-  
-  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  igraph_vector_ptr_t vids_by_type;
+  igraph_real_t maxcum, maxedges;
+
+  if (types < 1) IGRAPH_ERROR("types must be >= 1", IGRAPH_EINVAL);
+  if (nodes < 0) IGRAPH_ERROR("nodes must be >= 0", IGRAPH_EINVAL);
+  if (type_dist && igraph_vector_size(type_dist) != types) {
+    if (igraph_vector_size(type_dist) > types)
+      IGRAPH_WARNING("length of type_dist > types, type_dist will be trimmed");
+    else
+      IGRAPH_ERROR("type_dist vector too short", IGRAPH_EINVAL);
+  }
+  if (igraph_matrix_nrow(pref_matrix) < types ||
+      igraph_matrix_ncol(pref_matrix) < types)
+    IGRAPH_ERROR("pref_matrix too small", IGRAPH_EINVAL);
+
   IGRAPH_VECTOR_INIT_FINALLY(&cumdist, types+1);
 
   if (node_type_vec) {
@@ -1589,12 +1633,28 @@ int igraph_preference_game(igraph_t *graph, igraph_integer_t nodes,
     if (nodetypes == 0) {
       IGRAPH_ERROR("preference_game failed", IGRAPH_ENOMEM);
     }
+    IGRAPH_FINALLY(igraph_free, nodetypes);
     IGRAPH_VECTOR_INIT_FINALLY(nodetypes, nodes);
   }
-  
-  VECTOR(cumdist)[0]=0;
+
+  IGRAPH_CHECK(igraph_vector_ptr_init(&vids_by_type, types));
+  IGRAPH_FINALLY(igraph_vector_ptr_destroy_all, &vids_by_type);
   for (i=0; i<types; i++) {
-    VECTOR(cumdist)[i+1] = VECTOR(cumdist)[i]+VECTOR(*type_dist)[i];
+	  VECTOR(vids_by_type)[i] = igraph_Calloc(1, igraph_vector_t);
+	  if (VECTOR(vids_by_type)[i] == 0) {
+	    IGRAPH_ERROR("preference_game failed", IGRAPH_ENOMEM);
+	  }
+	  IGRAPH_CHECK(igraph_vector_init(VECTOR(vids_by_type)[i], 0));
+  }
+  IGRAPH_FINALLY_CLEAN(1);   /* removing igraph_vector_ptr_destroy_all */
+  IGRAPH_FINALLY(igraph_i_preference_game_free_vids_by_type, &vids_by_type);
+
+  VECTOR(cumdist)[0]=0;
+  if (type_dist) {
+    for (i=0; i<types; i++) 
+      VECTOR(cumdist)[i+1] = VECTOR(cumdist)[i]+VECTOR(*type_dist)[i];
+  } else {
+    for (i=0; i<types; i++) VECTOR(cumdist)[i+1] = i+1;
   }
   maxcum=igraph_vector_tail(&cumdist);
 
@@ -1605,33 +1665,110 @@ int igraph_preference_game(igraph_t *graph, igraph_integer_t nodes,
     igraph_real_t uni1=RNG_UNIF(0, maxcum);
     igraph_vector_binsearch(&cumdist, uni1, &type1);
     VECTOR(*nodetypes)[i] = type1-1;
+    IGRAPH_CHECK(igraph_vector_push_back(
+	    (igraph_vector_t*)VECTOR(vids_by_type)[type1-1], i));
   }
 
   igraph_vector_destroy(&cumdist);
   IGRAPH_FINALLY_CLEAN(1);
 
-  for (i=0; i<nodes; i++) {
-    for (j=(directed?0:i); j<nodes; j++) {
-      long type1, type2;
+  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&s, 0);
 
-      if (!loops && i == j) continue;
+  for (i=0; i<types; i++) {
+    for (j=0; j<types; j++) {
+      /* Generating the random subgraph between vertices of type i and j */
+      long int k, l;
+      igraph_real_t p, last;
+      igraph_vector_t *v1, *v2;
+      long int v1_size, v2_size;
+      
+      IGRAPH_ALLOW_INTERRUPTION();
 
-      type1=(long)VECTOR(*nodetypes)[i];
-      type2=(long)VECTOR(*nodetypes)[j];
+      v1 = (igraph_vector_t*)VECTOR(vids_by_type)[i];
+      v2 = (igraph_vector_t*)VECTOR(vids_by_type)[j];
+      v1_size = igraph_vector_size(v1);
+      v2_size = igraph_vector_size(v2);
 
-      if (RNG_UNIF01() < (igraph_real_t)MATRIX(*pref_matrix, type1, type2)) {
-	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
-	IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+      p = MATRIX(*pref_matrix, i, j);
+      igraph_vector_clear(&s);
+      if (i != j) {
+        /* The two vertex sets are disjoint, this is the easier case */
+        if (i > j && !directed) continue;
+        maxedges = v1_size * v2_size;
+      } else {
+        if (directed && loops) maxedges = v1_size * v1_size;
+        else if (directed && !loops) maxedges = v1_size * (v1_size-1);
+        else if (!directed && loops) maxedges = v1_size * (v1_size+1)/2;
+        else maxedges = v1_size * (v1_size-1)/2;
+      }
+
+      IGRAPH_CHECK(igraph_vector_reserve(&s, maxedges*p*1.1));
+        
+      last=RNG_GEOM(p);
+      while (last < maxedges) {
+        IGRAPH_CHECK(igraph_vector_push_back(&s, last));
+        last += RNG_GEOM(p);
+        last += 1;
+      }
+      l = igraph_vector_size(&s);
+        
+      IGRAPH_CHECK(igraph_vector_reserve(&edges, igraph_vector_size(&edges)+l*2));
+      
+      if (i != j) {
+        /* Generating the subgraph between vertices of type i and j */
+        for (k=0; k<l; k++) {
+          long int to=floor(VECTOR(s)[k]/v1_size);
+          long int from=VECTOR(s)[k]-((igraph_real_t)to)*v1_size;
+          igraph_vector_push_back(&edges, VECTOR(*v1)[from]);
+          igraph_vector_push_back(&edges, VECTOR(*v2)[to]);
+        }
+      } else {
+        /* Generating the subgraph among vertices of type i */
+        if (directed && loops) {
+          for (k=0; k<l; k++) {
+            long int to=floor(VECTOR(s)[k]/v1_size);
+            long int from=VECTOR(s)[k]-((igraph_real_t)to)*v1_size;
+            igraph_vector_push_back(&edges, VECTOR(*v1)[from]);
+            igraph_vector_push_back(&edges, VECTOR(*v1)[to]);
+          }
+        } else if (directed && !loops) {
+          for (k=0; k<l; k++) {
+            long int to=floor(VECTOR(s)[k]/v1_size);
+            long int from=VECTOR(s)[k]-((igraph_real_t)to)*v1_size;
+            if (from==to) to=v1_size-1;
+            igraph_vector_push_back(&edges, VECTOR(*v1)[from]); 
+            igraph_vector_push_back(&edges, VECTOR(*v1)[to]);
+          }
+        } else if (!directed && loops) {
+          for (k=0; k<l; k++) {
+            long int to=floor((sqrt(8*VECTOR(s)[k]+1)-1)/2);
+            long int from=VECTOR(s)[k]-(((igraph_real_t)to)*(to+1))/2;
+            igraph_vector_push_back(&edges, VECTOR(*v1)[from]);
+            igraph_vector_push_back(&edges, VECTOR(*v1)[to]);
+          }
+        } else {
+          for (k=0; k<l; k++) {
+            long int to=floor((sqrt(8*VECTOR(s)[k]+1)+1)/2);
+            long int from=VECTOR(s)[k]-(((igraph_real_t)to)*(to-1))/2;
+            igraph_vector_push_back(&edges, VECTOR(*v1)[from]);
+            igraph_vector_push_back(&edges, VECTOR(*v1)[to]);
+          }
+        }
       }
     }
   }
-  
+
   RNG_END();
-  
+
+  igraph_vector_destroy(&s);
+  igraph_i_preference_game_free_vids_by_type(&vids_by_type);
+  IGRAPH_FINALLY_CLEAN(2);
+
   if (node_type_vec == 0) {
     igraph_vector_destroy(nodetypes);
     igraph_Free(nodetypes);
-    IGRAPH_FINALLY_CLEAN(1);
+    IGRAPH_FINALLY_CLEAN(2);
   }
 
   IGRAPH_CHECK(igraph_create(graph, &edges, nodes, directed));
@@ -1657,8 +1794,10 @@ int igraph_preference_game(igraph_t *graph, igraph_integer_t nodes,
  * \param nodes The number of vertices in the graph.
  * \param types The number of vertex types.
  * \param type_dist_matrix Matrix giving the joint distribution of vertex types.
+ *   If null, incoming and outgoing vertex types are independent and uniformly
+ *   distributed.
  * \param pref_matrix Matrix giving the connection probabilities for
- * different vertex types.
+ *   different vertex types.
  * \param node_type_in_vec A vector where the individual generated "incoming"
  *   vertex types will be stored. If NULL, the vertex types won't be saved.
  * \param node_type_out_vec A vector where the individual generated "outgoing"
@@ -1668,8 +1807,9 @@ int igraph_preference_game(igraph_t *graph, igraph_integer_t nodes,
  *
  * Added in version 0.3.</para><para>
  *
- * Time complexity: O(|V|^2), |V| is the number of vertices.
- *
+ * Time complexity: O(|V|+|E|), the
+ * number of vertices plus the number of edges in the graph.
+ * 
  * \sa \ref igraph_preference_game()
  */
 
@@ -1682,13 +1822,26 @@ int igraph_asymmetric_preference_game(igraph_t *graph, igraph_integer_t nodes,
 				      igraph_bool_t loops) {
   
   long int i, j, k;
-  igraph_vector_t edges;
-  igraph_vector_t cumdist;
+  igraph_vector_t edges, cumdist, s, intersect;
   igraph_vector_t *nodetypes_in;
   igraph_vector_t *nodetypes_out;
-  igraph_real_t maxcum;
+  igraph_vector_ptr_t vids_by_intype, vids_by_outtype;
+  igraph_real_t maxcum, maxedges;
   
-  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  if (types < 1) IGRAPH_ERROR("types must be >= 1", IGRAPH_EINVAL);
+  if (nodes < 0) IGRAPH_ERROR("nodes must be >= 0", IGRAPH_EINVAL);
+  if (type_dist_matrix) {
+    if (igraph_matrix_nrow(type_dist_matrix) < types ||
+        igraph_matrix_ncol(type_dist_matrix) < types)
+      IGRAPH_ERROR("type_dist_matrix too small", IGRAPH_EINVAL);
+    else if (igraph_matrix_nrow(type_dist_matrix) > types ||
+        igraph_matrix_ncol(type_dist_matrix) > types)
+      IGRAPH_WARNING("type_dist_matrix will be trimmed");
+  }
+  if (igraph_matrix_nrow(pref_matrix) < types ||
+      igraph_matrix_ncol(pref_matrix) < types)
+    IGRAPH_ERROR("pref_matrix too small", IGRAPH_EINVAL);
+
   IGRAPH_VECTOR_INIT_FINALLY(&cumdist, types*types+1);
 
   if (node_type_in_vec) {
@@ -1713,45 +1866,129 @@ int igraph_asymmetric_preference_game(igraph_t *graph, igraph_integer_t nodes,
     IGRAPH_VECTOR_INIT_FINALLY(nodetypes_out, nodes);
   }
    
+  IGRAPH_CHECK(igraph_vector_ptr_init(&vids_by_intype, types));
+  IGRAPH_FINALLY(igraph_vector_ptr_destroy_all, &vids_by_intype);
+  IGRAPH_CHECK(igraph_vector_ptr_init(&vids_by_outtype, types));
+  IGRAPH_FINALLY(igraph_vector_ptr_destroy_all, &vids_by_outtype);
+  for (i=0; i<types; i++) {
+	  VECTOR(vids_by_intype)[i] = igraph_Calloc(1, igraph_vector_t);
+	  VECTOR(vids_by_outtype)[i] = igraph_Calloc(1, igraph_vector_t);
+	  if (VECTOR(vids_by_intype)[i] == 0 || VECTOR(vids_by_outtype)[i] == 0) {
+	    IGRAPH_ERROR("asymmetric_preference_game failed", IGRAPH_ENOMEM);
+	  }
+	  IGRAPH_CHECK(igraph_vector_init(VECTOR(vids_by_intype)[i], 0));
+	  IGRAPH_CHECK(igraph_vector_init(VECTOR(vids_by_outtype)[i], 0));
+  }
+  IGRAPH_FINALLY_CLEAN(2);   /* removing igraph_vector_ptr_destroy_all */
+  IGRAPH_FINALLY(igraph_i_preference_game_free_vids_by_type, &vids_by_intype);
+  IGRAPH_FINALLY(igraph_i_preference_game_free_vids_by_type, &vids_by_outtype);
+
   VECTOR(cumdist)[0]=0;
-  for (i=0, k=0; i<types; i++) {
-    for (j=0; j<types; j++, k++) {
-      VECTOR(cumdist)[k+1] = VECTOR(cumdist)[k]+MATRIX(*type_dist_matrix, i, j);
+  if (type_dist_matrix) {
+    for (i=0, k=0; i<types; i++) {
+      for (j=0; j<types; j++, k++) {
+        VECTOR(cumdist)[k+1] = VECTOR(cumdist)[k]+MATRIX(*type_dist_matrix, i, j);
+      }
     }
+  } else {
+    for (i=0; i<types*types; i++) VECTOR(cumdist)[i+1] = i+1;
   }
   maxcum=igraph_vector_tail(&cumdist);
 
   RNG_BEGIN();
 
   for (i=0; i<nodes; i++) {
-    long int type1;
+    long int type1, type2;
     igraph_real_t uni1=RNG_UNIF(0, maxcum);
     igraph_vector_binsearch(&cumdist, uni1, &type1);
-    VECTOR(*nodetypes_in)[i] = (type1-1) / (int)types;
-    VECTOR(*nodetypes_out)[i] = (type1-1) % (int)types;
+    type2 = (type1-1) % (int)types;
+    type1 = (type1-1) / (int)types;
+    VECTOR(*nodetypes_in)[i] = type1; 
+    VECTOR(*nodetypes_out)[i] = type2; 
+    IGRAPH_CHECK(igraph_vector_push_back(
+	    (igraph_vector_t*)VECTOR(vids_by_intype)[type1], i));
+    IGRAPH_CHECK(igraph_vector_push_back(
+	    (igraph_vector_t*)VECTOR(vids_by_outtype)[type2], i));
   }
 
   igraph_vector_destroy(&cumdist);
   IGRAPH_FINALLY_CLEAN(1);
 
-  for (i=0; i<nodes; i++) {
-    for (j=0; j<nodes; j++) {
-      long type1, type2;
+  IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&s, 0);
+  IGRAPH_VECTOR_INIT_FINALLY(&intersect, 0);
+  for (i=0; i<types; i++) {
+    for (j=0; j<types; j++) {
+      long int k, l, c;
+      igraph_real_t p, last;
+      igraph_vector_t *v1, *v2;
+      long int v1_size, v2_size;
 
-      if (!loops && i == j) continue;
+      IGRAPH_ALLOW_INTERRUPTION();
+      
+      v1 = (igraph_vector_t*)VECTOR(vids_by_outtype)[i];
+      v2 = (igraph_vector_t*)VECTOR(vids_by_intype)[j];
+      v1_size = igraph_vector_size(v1);
+      v2_size = igraph_vector_size(v2);
 
-      type1=(long)VECTOR(*nodetypes_out)[i];
-      type2=(long)VECTOR(*nodetypes_in)[j];
+      maxedges = v1_size * v2_size;
+      if (!loops) {
+        IGRAPH_CHECK(igraph_vector_intersect_sorted(v1, v2, &intersect, 0));
+        c = igraph_vector_size(&intersect);
+        maxedges -= c; 
+      }
 
-      if (RNG_UNIF01() < MATRIX(*pref_matrix, type1, type2)) {
-	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
-	IGRAPH_CHECK(igraph_vector_push_back(&edges, j));
+      p = MATRIX(*pref_matrix, i, j);
+      igraph_vector_clear(&s);
+      IGRAPH_CHECK(igraph_vector_reserve(&s, maxedges*p*1.1));
+        
+      last=RNG_GEOM(p);
+      while (last < maxedges) {
+        IGRAPH_CHECK(igraph_vector_push_back(&s, last));
+        last += RNG_GEOM(p);
+        last += 1;
+      }
+      l = igraph_vector_size(&s);
+        
+      IGRAPH_CHECK(igraph_vector_reserve(&edges, igraph_vector_size(&edges)+l*2));
+
+      if (!loops && c>0) {
+          for (k=0; k<l; k++) {
+            long int to=floor(VECTOR(s)[k]/v1_size);
+            long int from=VECTOR(s)[k]-((igraph_real_t)to)*v1_size;
+            if (VECTOR(*v1)[from] == VECTOR(*v2)[to]) {
+              /* remap loop edges */
+              to = v2_size-1;
+              igraph_vector_binsearch(&intersect, VECTOR(*v1)[from], &c);
+              from = v1_size-1;
+              if (VECTOR(*v1)[from] == VECTOR(*v2)[to]) from--;
+              while (c>0) {
+                c--; from--;
+                if (VECTOR(*v1)[from] == VECTOR(*v2)[to]) from--;
+              }
+            }
+            igraph_vector_push_back(&edges, VECTOR(*v1)[from]); 
+            igraph_vector_push_back(&edges, VECTOR(*v2)[to]);
+          }
+      } else {
+        for (k=0; k<l; k++) {
+          long int to=floor(VECTOR(s)[k]/v1_size);
+          long int from=VECTOR(s)[k]-((igraph_real_t)to)*v1_size;
+          igraph_vector_push_back(&edges, VECTOR(*v1)[from]);
+          igraph_vector_push_back(&edges, VECTOR(*v2)[to]);
+        }
       }
     }
   }
-  
+
   RNG_END();
   
+  igraph_vector_destroy(&s);
+  igraph_vector_destroy(&intersect);
+  igraph_i_preference_game_free_vids_by_type(&vids_by_intype);
+  igraph_i_preference_game_free_vids_by_type(&vids_by_outtype);
+  IGRAPH_FINALLY_CLEAN(4);
+
   if (node_type_out_vec == 0) {
     igraph_vector_destroy(nodetypes_out);
     igraph_Free(nodetypes_out);
