@@ -92,6 +92,10 @@ int igraph_empty_attrs(igraph_t *graph, igraph_integer_t n, igraph_bool_t direct
     IGRAPH_ERROR("cannot create empty graph with negative number of vertices",
 		  IGRAPH_EINVAL);
   }
+  
+  if (!IGRAPH_FINITE(n)) {
+    IGRAPH_ERROR("number of vertices is not finite (NA, NaN or Inf)", IGRAPH_EINVAL);
+  }
 
   graph->n=0;
   graph->directed=directed;
